<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:saml="urn:oasis:names:tc:SAML:1.0:assertion" exclude-result-prefixes="saml">
	<xsl:output method="xml" version="1.0" encoding="UTF-8" indent="yes"/>
	
	<!-- declare a global variable -->
	<xsl:variable name="SAML-URI">urn:oasis:names:tc:SAML:1.0:assertion</xsl:variable>
	
	<!-- the main template that emit all the Attribute elements -->
	<xsl:template name="emitAttributeElement">
		<xsl:param name="name"/>
		<xsl:param name="type" select="$SAML-URI"/>
		<xsl:param name="subelement" select="."/>
		<xsl:element name="stsuuser:Attribute" namespace="urn:ibm:names:ITFIM:1.0:stsuuser">
			<xsl:attribute name="name"><xsl:value-of select="$name"/></xsl:attribute>
			<xsl:attribute name="type"><xsl:value-of select="$type"/></xsl:attribute>
			<xsl:for-each select="$subelement">
				<xsl:element name="stsuuser:Value" namespace="urn:ibm:names:ITFIM:1.0:stsuuser">
					<xsl:choose>
						<xsl:when test="count(node()) &gt; 1">
						 	<xsl:copy-of select="./*"/>
						</xsl:when>
						<xsl:otherwise>
							<xsl:value-of select="."/>
						</xsl:otherwise> 
					</xsl:choose>
				</xsl:element>
			</xsl:for-each>
		</xsl:element>
	</xsl:template>
	
	<!-- Matching the doument element -->
	<xsl:template match="saml:Assertion">
		<xsl:element name="stsuuser:STSUniversalUser" namespace="urn:ibm:names:ITFIM:1.0:stsuuser">
			<xsl:attribute name="version">1.0</xsl:attribute>
			<xsl:element name="stsuuser:Principal" namespace="urn:ibm:names:ITFIM:1.0:stsuuser">
				<xsl:apply-templates select="saml:AuthenticationStatement/saml:Subject/*"/>
			</xsl:element>
			<xsl:element name="stsuuser:AttributeList" namespace="urn:ibm:names:ITFIM:1.0:stsuuser">
				<xsl:for-each select="@*[string-length(namespace-uri()) = 0]">
					<xsl:call-template name="emitAttributeElement">
						<xsl:with-param name="name" select="name()"/>
					</xsl:call-template>
				</xsl:for-each>
				<xsl:apply-templates select="*"/>
			</xsl:element>
		</xsl:element>
	</xsl:template>
	
	<!--Emit the Principal element content -->
	<xsl:template match="saml:NameIdentifier">
		<xsl:call-template name="emitAttributeElement">
			<xsl:with-param name="name">name</xsl:with-param>
			<xsl:with-param name="type" select="@Format"/>
		</xsl:call-template>
		<xsl:call-template name="emitAttributeElement">
			<xsl:with-param name="name">NameQualifier</xsl:with-param>
			<xsl:with-param name="type">urn:oasis:names:tc:SAML:1.0:assertion</xsl:with-param>
			<xsl:with-param name="subelement" select="@NameQualifier"/>
		</xsl:call-template>			
	</xsl:template>
	
	<!-- Conditions and AuthenticationStatement are treated the same way -->
	<xsl:template match="saml:Conditions | saml:AuthenticationStatement">
		<xsl:for-each select="@*[string-length(namespace-uri()) = 0]">
			<xsl:call-template name="emitAttributeElement">
				<xsl:with-param name="name" select="name()"/>
			</xsl:call-template>
		</xsl:for-each>
	</xsl:template>
	<xsl:template match="saml:Conditions">
		<xsl:apply-templates select="*"/>
	</xsl:template>
	
	<xsl:template match="saml:AudienceRestrictionCondition">
		<xsl:if test="count(*[local-name() = 'Audience']) > 0">
			<xsl:call-template name="emitAttributeElement">
				<xsl:with-param name="name">AudienceRestriction</xsl:with-param>
				<xsl:with-param name="subelement" select="saml:Audience"/>
			</xsl:call-template>
		</xsl:if>
	</xsl:template>
	
	<!-- Iterate through the Attribute elements -->
	<xsl:template match="saml:AttributeStatement">
		<xsl:apply-templates select="saml:Attribute"/>
	</xsl:template>
	
	<xsl:template match="saml:Attribute">
		<xsl:call-template name="emitAttributeElement">
			<xsl:with-param name="name" select="@AttributeName"/>
			<xsl:with-param name="type" select="@AttributeNamespace"/>
			<xsl:with-param name="subelement" select="saml:AttributeValue"/>
		</xsl:call-template>
	</xsl:template>
	
</xsl:stylesheet>
